@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -61,9 +61,9 @@
 #define MAV_2_PI 6.2831853
 #define MAV_DEG2RAD(X) ((X)*MAV_PI_OVER_180)
 #define MAV_RAD2DEG(X) ((X)*MAV_180_OVER_PI)
-#define MAV_MATRIX_XCOMP 3][0
-#define MAV_MATRIX_YCOMP 3][1
-#define MAV_MATRIX_ZCOMP 3][2
+#define MAV_MATRIX_XCOMP 0][3
+#define MAV_MATRIX_YCOMP 1][3
+#define MAV_MATRIX_ZCOMP 2][3
 
 #define MAV_REDEFINE_WARN 1
 #define MAV_REDEFINE_NOWARN 2
@@ -250,7 +250,7 @@ typedef struct {
 typedef struct {
   float offset;
   float angle;
-  void *usedef;
+  void *userdef;
 } MAV_viewModifierParams;
 
 /* Clip planes */
@@ -375,6 +375,8 @@ void mav_windowBackgroundColourSet(MAV_window *w, float r, float g, float b);
 void mav_windowBackfaceCullSet(MAV_window *w, int v);
 int  mav_windowBackfaceCullGet(MAV_window *w);
 void mav_windowBlendSet(MAV_window *w, int v);
+void mav_windowLineWidthSet(MAV_window *w, float v);
+void mav_windowLineStippleSet(MAV_window *w, int factor, unsigned short pattern);
 void mav_windowPerspectiveSet(MAV_window *w, float ncp, float fcp, float fov, float aspect);
 void mav_windowPolygonModeSet(MAV_window *w, int v);
 void mav_windowViewParamsSet(MAV_window *w, MAV_viewParams *vp);
@@ -465,6 +467,7 @@ MAV_vector mav_vectorWorldPos(MAV_vector v);
 void mav_vectorPrint(char *s, MAV_vector v);
 
 MAV_matrix mav_matrixSet(float roll, float pitch, float yaw, float x, float y, float z);
+MAV_matrix mav_matrixSetOld(float roll, float pitch, float yaw, float x, float y, float z);
 MAV_matrix mav_matrixXYZSet(MAV_matrix m, MAV_vector v);
 MAV_matrix mav_matrixRPYSet(MAV_matrix m, float roll, float pitch, float yaw);
 MAV_vector mav_matrixXYZGet(MAV_matrix m);
