@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -47,6 +47,7 @@ int mav_opt_shareContexts= MAV_TRUE;
 int mav_opt_trackMatrix= MAV_FALSE;
 
 int mavlib_matrixmode= MAV_MODELVIEW;
+int mavlib_voodoo= MAV_FALSE;
 
 
 
@@ -134,6 +135,7 @@ void mav_gfxBackgroundColourSet(float r, float g, float b)
   mavlib_bk[3]= 0.0;
 #else
   glClearColor(r,g,b,0.0);
+  glClearAccum(r,g,b,0.0);
 #endif
 #endif
 }
@@ -252,9 +254,24 @@ void mav_gfxMatrixPop(void)
 #endif
 }
 
-void mav_gfxMatrixMult(MAV_matrix m)
+MAV_matrix mavlib_matrixTranspose(MAV_matrix mat)
+{
+  MAV_matrix rv;
+  int i,j;
+  
+  for (i=0; i<4; i++) {
+    for (j=0; j<4; j++) {
+      rv.mat[i][j]= mat.mat[j][i];
+    }
+  }
+
+  return rv;
+}
+
+void mav_gfxMatrixMult(MAV_matrix im)
 {
 #ifndef MAV_NONE
+  MAV_matrix m= mavlib_matrixTranspose(im);
 #ifdef MAV_IRISGL
 #ifdef MAV_IRIX5
   multmatrix(m.mat);
@@ -299,12 +316,13 @@ MAV_matrix mav_gfxMatrixGet(void)
 #endif
 #endif
 
-  return rv;
+  return mavlib_matrixTranspose(rv);
 }
 
-void mav_gfxMatrixLoad(MAV_matrix mat)
+void mav_gfxMatrixLoad(MAV_matrix imat)
 {
 #ifndef MAV_NONE
+  MAV_matrix mat= mavlib_matrixTranspose(imat);
 #ifdef MAV_IRISGL
 #ifdef MAV_IRIX5
   loadmatrix(mat.mat);
@@ -1145,6 +1163,24 @@ float mav_gfxLineWidthGet(void)
   return rv;
 }
 
+void mav_gfxLineStippleSet(int factor, unsigned short pattern)
+{
+#ifndef MAV_NONE
+#ifdef MAV_IRISGL
+#else
+  if (factor>0)
+  {
+    glLineStipple(factor, pattern);
+    glEnable(GL_LINE_STIPPLE);
+  }
+  else
+  {
+    glDisable(GL_LINE_STIPPLE);
+  }
+#endif
+#endif
+}
+
 void mav_gfxBlendSet(int v)
 {
 #ifndef MAV_NONE
@@ -1180,10 +1216,31 @@ void mav_gfxAccumSet(int mode, float val)
 #endif
     break;
 
+  case MAV_ACCUM_LOAD:
+#ifdef MAV_IRISGL
+#else
+    glAccum(GL_LOAD, val);
+#endif
+    break;
+
   case MAV_ACCUM_RETURN:
 #ifdef MAV_IRISGL
 #else
     glAccum(GL_RETURN, val);
+#endif
+    break;
+
+  case MAV_ACCUM_ADD:
+#ifdef MAV_IRISGL
+#else
+    glAccum(GL_ADD, val);
+#endif
+    break;
+
+  case MAV_ACCUM_MULT:
+#ifdef MAV_IRISGL
+#else
+    glAccum(GL_MULT, val);
 #endif
     break;
   }
