@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -35,6 +35,8 @@ void mav_moduleNew(char *fn(void));
 extern int mav_opt_bindTextures;
 extern int mav_opt_shareContexts;
 extern int mav_opt_maxTextures;
+extern int mavlib_voodoo;
+extern void *mavlib_dlh;
 
 
 
@@ -43,6 +45,7 @@ extern int mav_opt_maxTextures;
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <dlfcn.h>
 
 #ifndef MAV_NONE
 #include <X11/keysym.h>
@@ -52,12 +55,6 @@ extern int mav_opt_maxTextures;
 #include <gl/glws.h>
 #else
 #include <GL/glx.h>
-#ifdef MAV_LINUX
-#ifdef MAV_VOODOO
-#include "GL/xmesa.h"
-extern GLboolean fxMesaSelectCurrentBoard(int);
-#endif
-#endif
 #ifdef GL_EXT_texture_object
 extern GLuint *mavlib_bindTextureIndex;
 #endif
@@ -116,11 +113,14 @@ char *mav_gfxModuleID(void)
 #ifdef MAV_IRISGL
   return "Graphics (IrisGL and X11)";
 #else
-#ifdef MAV_VOODOO
-  return "Graphics (OpenGL and X11 with Voodoo options)";
-#else
-  return "Graphics (OpenGL and X11)";
-#endif
+  if (mavlib_voodoo)
+  {
+    return "Graphics (OpenGL and X11, Voodoo detected)";
+  }
+  else
+  {
+    return "Graphics (OpenGL and X11)";
+  }
 #endif
 #endif
 }
@@ -149,6 +149,7 @@ void mavlib_gfxExit()
 
 int mav_gfxModuleInit()
 {
+  char *voodoo;
 #ifndef MAV_NONE
   int i;
   /* Initialise data structure */
@@ -176,6 +177,13 @@ int mav_gfxModuleInit()
   mavlib_rootwin= RootWindow(mavlib_dpy, mavlib_scrnum);
 #endif
 
+  /* use environment variable to look for a Voodoo card */
+  voodoo= getenv("MESA_GLX_FX");
+  if (voodoo) {
+    if (!strcmp(voodoo, "f")) mavlib_voodoo= 1;
+    if (!strcmp(voodoo, "fullscreen")) mavlib_voodoo= 1;
+  }
+
   return 1;
 }
 
@@ -943,31 +951,48 @@ int mav_gfxWindowKeyGet(int key)
 
 void mav_gfx3DfxModeSet(int fullscreen)
 {
-#ifdef MAV_OPENGL
-#ifdef MAV_LINUX
-#ifdef MAV_VOODOO
-  if (fullscreen)
+  typedef unsigned char (*FN)(int);
+  FN fn;
+  
+  /* Look for XMesaSetFXmode function */
+  fn= (FN) dlsym(mavlib_dlh, "XMesaSetFXmode");
+
+  if (fn)
   {
-    XMesaSetFXmode(XMESA_FX_FULLSCREEN);  
+    /* Execute function with correct parameters - may change in future */
+    if (fullscreen)
+    {
+      fn(2);
+    }
+    else
+    {
+      fn(1);
+    }
   }
   else
   {
-    XMesaSetFXmode(XMESA_FX_WINDOW);  
+    fprintf(stderr, "Warning: cound not find XMesaSetFXmode function, ignoring\n");
   }
-#endif
-#endif
-#endif
 }
 
 int mav_gfx3DfxBoardSet(int bd)
 {
-#ifdef MAV_OPENGL
-#ifdef MAV_LINUX
-#ifdef MAV_VOODOO
-  return fxMesaSelectCurrentBoard(bd);
-#endif
-#endif
-#endif
+  typedef unsigned char (*FN)(int);
+  FN fn;
+  int rv=0;
+  
+  /* Look for fxMesaSelectCurrentBoard function */
+  fn= (FN) dlsym(mavlib_dlh, "fxMesaSelectCurrentBoard");
 
-  return 0;
+  if (fn) 
+  {
+    /* Execute function with correct parameters - may change in future */
+    rv= fn(bd);
+  }
+  else
+  {
+    fprintf(stderr, "Warning: cound not find fxMesaSelectCurrentBoard function, ignoring\n");
+  }
+
+  return rv;
 }
