@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -260,8 +260,9 @@ void mav_frameBegin(void)
   while (mav_listItemNext(mav_win_list, (void **) &w)) {
     if (w!=mav_win_current) mav_windowSet(w);
 
-    /* clear colour and depth buffers */
+    /* clear colour, depth and, if enabled, accumulation buffers */
     mav_gfxClearCZ();
+    if (mav_opt_accumBuf) mav_gfxClearA();
     mav_surfaceParamsUndefine();
     
     /* calculate view matrix orientation */
@@ -294,17 +295,17 @@ void mav_frameBegin(void)
     /* define and load the orientation part of the matrix */
 
     view_matrix.mat[0][0]= w->right.x;
-    view_matrix.mat[0][1]= w->up.x;
-    view_matrix.mat[0][2]= -w->view.x;
-    view_matrix.mat[0][3]= 0.0;
+    view_matrix.mat[0][1]= w->right.y;
+    view_matrix.mat[0][2]= w->right.z;
+    view_matrix.mat[0][3]= 0;
 
-    view_matrix.mat[1][0]= w->right.y;
+    view_matrix.mat[1][0]= w->up.x;
     view_matrix.mat[1][1]= w->up.y;
-    view_matrix.mat[1][2]= -w->view.y;
-    view_matrix.mat[1][3]= 0.0;
+    view_matrix.mat[1][2]= w->up.z;
+    view_matrix.mat[1][3]= 0;
 
-    view_matrix.mat[2][0]= w->right.z;
-    view_matrix.mat[2][1]= w->up.z;
+    view_matrix.mat[2][0]= -w->view.x;
+    view_matrix.mat[2][1]= -w->view.y;
     view_matrix.mat[2][2]= -w->view.z;
     view_matrix.mat[2][3]= 0.0;
 
@@ -312,6 +313,7 @@ void mav_frameBegin(void)
     view_matrix.mat[3][1]= 0;
     view_matrix.mat[3][2]= 0;
     view_matrix.mat[3][3]= 1.0;
+
     mav_gfxMatrixLoad(view_matrix);
 
     /* account for eye position */ 
