@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -31,6 +31,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
+#include <sys/types.h>
+#include <unistd.h>
 char *strdup (const char *s1);
 
 MAV_surfaceParams mavlib_surfaceParams={-1,-1,-1,-1};
@@ -110,8 +112,7 @@ void mav_surfaceParamsUse(MAV_surfaceParams *sp)
     switch (sp->mode) {
     case MAV_COLOUR:
       if (sp->colour!=mavlib_surfaceParams.colour) {
-        if (sp->colour<0) /* build in colours */
-          {
+        if (sp->colour<0) { /* build in colours */
   	  switch (sp->colour) {
   	  case MAV_COLOUR_BLACK:
   	    mav_gfxColourUse(mavlib_colBlack);
@@ -402,69 +403,9 @@ void mav_paletteMaterialSet(MAV_palette *p, int index, float a1, float a2, float
   }
 }
 
-#ifdef MAV_TIFF
-#include <tiffio.h>
-#endif
-
-/* Routine to read a texture from a tiff file */
-
-int mavlib_readTiff(char *filename, int *width, int *height, unsigned long **mem)
-{
-#ifdef MAV_TIFF
-  TIFF *tif;
-  TIFFRGBAImage img;
-  unsigned long *map;
-  char emsg[1024];
-
-/* Open tiff file */
-
-  tif = TIFFOpen(filename, "r");
-  if (tif == NULL) return(MAV_FALSE);
-
-/* Parse */
-
-  if (TIFFRGBAImageBegin(&img, tif, 0, emsg))
-    {
-      *width  = (int) img.width;
-      *height = (int) img.height;
-      *mem = mav_malloc(*width**height * sizeof(unsigned long));
-      map=*mem;
-
-      if (TIFFRGBAImageGet(&img, map, img.width, img.height) == 0) return(MAV_FALSE);
-
-      /* TIFF library read in ABGR. However, Linux needs a byte-order swap */
-
-#ifdef MAV_LINUX
-      {
-        int i,b1,b2,b3,b4;
-
-        for (i=0; i<*height**width; i++) {
-	  b1=(int) (map[i] & 0xff);
-	  b2=(int) ((map[i] >> 8) & 0xff);
-	  b3=(int) ((map[i] >> 16) & 0xff);
-	  b4=(int) ((map[i] >> 24) & 0xff);
-	  
-	  map[i]= (b1<<24) + (b2<<16) + (b3<<8) + b4;
-        }
-      }
-#endif
-      TIFFRGBAImageEnd(&img);
-    }
-  else 
-    {
-      return(MAV_FALSE);
-    }
-
-  return(MAV_TRUE);
-#else
-  if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: TIFF format not supported on this platform, ignoring\n");
-  return(MAV_FALSE);
-#endif
-}
 
 
-
-/* Routine to read a texture from a raw or ascii PPM file */
+/* Routine to read a texture from a raw or ASCII PPM file */
 
 int mavlib_readPPM(char *filename, int *width, int *height, unsigned long **mem)
 {
@@ -575,12 +516,36 @@ int mav_paletteTextureSet(MAV_palette *p, int index, char *filename)
     
     if (p->texlist[index].defined==MAV_REDEFINE_WARN && mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: Texture index %i already defined in palette, overwriting\n", index);
     
-    /* Use appropriate read for file extension type */
+    /* We can read PPM directly, otherwise use ImageMagick's covert program */
     
-    if (strstr(filename, ".pnm") || strstr(filename, ".ppm")) rv= mavlib_readPPM(filename, &p->texlist[index].width, &p->texlist[index].height, &p->texlist[index].mem);
-    else if (strstr(filename, ".tif") || strstr(filename, ".tiff")) rv= mavlib_readTiff(filename, &p->texlist[index].width, &p->texlist[index].height, &p->texlist[index].mem);
-    else {
-      if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: file format for %s not recognised, ignoring\n", filename);
+    if (strstr(filename, ".pnm") || strstr(filename, ".ppm")) 
+    {
+      rv= mavlib_readPPM(filename, &p->texlist[index].width, &p->texlist[index].height, &p->texlist[index].mem);
+    }
+    else 
+    {
+      char buf[500], f1[500], f2[500];
+      int srv;
+
+      sprintf(f1, "/tmp/mavtex%i.ppm", getpid());
+      sprintf(f2, "/tmp/mavtexlog%i", getpid());
+
+      sprintf(buf, "convert %s %s 2>%s", filename, f1, f2);
+      srv= system(buf);
+
+      if (srv==0)
+      {
+	rv= mavlib_readPPM(f1, &p->texlist[index].width, &p->texlist[index].height, &p->texlist[index].mem);
+	if (rv==MAV_TRUE) {
+	  sprintf(buf, "rm -f %s %s", f1, f2);
+	  system(buf);
+	}
+      }
+      else
+      {
+	if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: can not read texture file %s, ignoring\n", filename);	
+	rv= MAV_FALSE;
+      }
     }
     
     /* Only interger powers of 2 allowed for height and width */
