@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -39,11 +39,13 @@ char *strdup (const char *s1);
 
 int mavlib_ac3d_matNo; 
 int *mavlib_ac3d_matLookUp;
+int *mavlib_ac3d_colLookUp;
 FILE *mavlib_ac3d_file;
 int mavlib_ac3d_source;
 char *mavlib_ac3d_str;
 int mavlib_ac3d_strCount;
 int mavlib_ac3d_strLen;
+int mavlib_ac3d_nl;
 
 typedef struct {
   int numverts;
@@ -63,6 +65,22 @@ typedef struct {
 
 
 
+/* Routine to print an AC3D error/warning message */
+
+void mavlib_ac3d_fprintf(char *s)
+{
+  /* Do we need to print a new line */
+  if (!mavlib_ac3d_nl) {
+    fprintf(stderr, "\n");
+    mavlib_ac3d_nl=1;
+  }
+
+  /* Print error/warning message */
+  fprintf(stderr, s);
+}
+
+
+
 /* Routine to return an empty or matching material index */
 
 int mavlib_ac3d_findMatPlace(float *dif, float *amb, float *emis, float *spec, float shin, float trans)
@@ -103,7 +121,40 @@ int mavlib_ac3d_findMatPlace(float *dif, float *amb, float *emis, float *spec, f
   }
   
   /* cant find empty slot */
-  fprintf(stderr, "\nError: maximum number of materials exceeded.\n");
+  mavlib_ac3d_fprintf("Error: maximum number of materials exceeded.\n");
+  return -1;
+}
+
+
+
+/* Routine to return an empty or matching colour index */
+
+int mavlib_ac3d_findColPlace(float *amb, float trans)
+{
+  int i;
+
+  trans=1.0-trans;
+
+  /* check if colour has already been defined */
+  for (i=0; i<mav_opt_maxColours; i++) {
+    if (mav_palette_default->collist[i].defined) {
+      if (mav_palette_default->collist[i].colour[0]==amb[0] &&
+	  mav_palette_default->collist[i].colour[1]==amb[1] &&
+	  mav_palette_default->collist[i].colour[2]==amb[2] &&
+	  mav_palette_default->collist[i].colour[3]==trans) return i;
+    }
+  }
+
+  /* Not found, so need to define it - first find an unused colour */
+  for (i=0; i<mav_opt_maxColours; i++) {
+    if (!mav_palette_default->collist[i].defined) {
+      mav_paletteColourSet(mav_palette_default, i, amb[0], amb[1], amb[2], trans);
+      return i;
+    }
+  }
+
+  /* cant find empty slot */
+  mavlib_ac3d_fprintf("Error: maximum number of colours exceeded.\n");
   return -1;
 }
 
@@ -126,13 +177,12 @@ int mavlib_ac3d_texLookUp(char *nm)
   for (i=0; i<mav_opt_maxTextures; i++) {
     if (!mav_palette_default->texlist[i].defined) {
       if (mav_paletteTextureSet(mav_palette_default, i, nm)==MAV_FALSE) return -1;
-      if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "added texture %s\n", nm);
       return i;
     }
   }
   
   /* cant find empty slot */
-  fprintf(stderr, "Error: maximum number of textures exceeded.\n");
+  mavlib_ac3d_fprintf("Error: maximum number of textures exceeded.\n");
   return -1;
 }
 
@@ -182,8 +232,6 @@ void mavlib_ac3d_parseString(char *str)
       for (i=1; i<strlen(str)-1; i++) str[i-1]=str[i];
       str[i-1]=0;
     }
-
-    /*printf("got %s l %i\n", str, strlen(str));*/
   }
 }
 
@@ -326,10 +374,15 @@ int mavlib_ac3d_parseMaterial(void)
   /* give material an index taking into account duplicates enteries */
   mavlib_ac3d_matLookUp[mavlib_ac3d_matNo]= mavlib_ac3d_findMatPlace(dif, amb, emis, spec, shin, trans);
 
+  /* give material a colour index taking into account duplicates enteries */
+  mavlib_ac3d_colLookUp[mavlib_ac3d_matNo]= mavlib_ac3d_findColPlace(amb, trans);
+
   /* Check if there was room for the material */
   if (mavlib_ac3d_matLookUp[mavlib_ac3d_matNo]==-1) return -1;
+  if (mavlib_ac3d_colLookUp[mavlib_ac3d_matNo]==-1) return -1;
 
   mavlib_ac3d_matNo++;
+
   return 0;
 }
 
@@ -379,23 +432,23 @@ int mavlib_ac3d_parseObject(MAV_facet *fc, MAV_polyline *pl, MAV_matrix mat)
     /* rotation */
     if (!strcmp(str, "rot")) {
       mavlib_ac3d_parseFloat(&omat.mat[0][0]);
-      mavlib_ac3d_parseFloat(&omat.mat[0][1]);
-      mavlib_ac3d_parseFloat(&omat.mat[0][2]);
-
       mavlib_ac3d_parseFloat(&omat.mat[1][0]);
-      mavlib_ac3d_parseFloat(&omat.mat[1][1]);
-      mavlib_ac3d_parseFloat(&omat.mat[1][2]);
-
       mavlib_ac3d_parseFloat(&omat.mat[2][0]);
+
+      mavlib_ac3d_parseFloat(&omat.mat[0][1]);
+      mavlib_ac3d_parseFloat(&omat.mat[1][1]);
       mavlib_ac3d_parseFloat(&omat.mat[2][1]);
+
+      mavlib_ac3d_parseFloat(&omat.mat[0][2]);
+      mavlib_ac3d_parseFloat(&omat.mat[1][2]);
       mavlib_ac3d_parseFloat(&omat.mat[2][2]);
     }
 
     /* location */
     if (!strcmp(str, "loc")) {
-      mavlib_ac3d_parseFloat(&omat.mat[3][0]);
-      mavlib_ac3d_parseFloat(&omat.mat[3][1]);
-      mavlib_ac3d_parseFloat(&omat.mat[3][2]);
+      mavlib_ac3d_parseFloat(&omat.mat[0][3]);
+      mavlib_ac3d_parseFloat(&omat.mat[1][3]);
+      mavlib_ac3d_parseFloat(&omat.mat[2][3]);
     }
 
     /* URL */
@@ -455,37 +508,39 @@ int mavlib_ac3d_parseObject(MAV_facet *fc, MAV_polyline *pl, MAV_matrix mat)
 	  mavlib_ac3d_parseFloat(&obj->tex[i][j].t);
 	}
 
-	/* calculate flat shaded surface normal */
-	if (obj->refs[i]>2)
-	{
-	  j=0;
-	  do {
-	    if (j==obj->refs[i]-2)
-	    {
-	      if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: can't find surface normal\n");
-	      obj->refs[i]=0;
-	      v1.x= 1;
-	      v1.y= 0;
-	      v1.z= 0;
-	      v2.x= 0;
-	      v2.y= 1;
-	      v2.z= 0;
-	    }
-	    else
-	    {
-	      v1= mav_vectorNormalize(mav_vectorSub(obj->verts[obj->index[i][j]], obj->verts[obj->index[i][j+1]]));
-	      v2= mav_vectorNormalize(mav_vectorSub(obj->verts[obj->index[i][j+2]], obj->verts[obj->index[i][j+1]]));	    
-	    }
-	    j++;
-	  } while (fabs(mav_vectorDotProduct(v1, v2))==1.0);
+	/* calculate flat shaded surface normal for polygon */
+	if ((obj->surfprops[i] & 0xF)==0) {
+	  if (obj->refs[i]>2)
+	  {
+	    j=0;
+	    do {
+	      if (j==obj->refs[i]-2)
+	      {
+		if (mav_opt_output==MAV_VERBOSE) mavlib_ac3d_fprintf("Warning: can't find surface normal.\n");
+		obj->refs[i]=0;
+		v1.x= 1;
+		v1.y= 0;
+		v1.z= 0;
+		v2.x= 0;
+		v2.y= 1;
+		v2.z= 0;
+	      }
+	      else
+	      {
+		v1= mav_vectorNormalize(mav_vectorSub(obj->verts[obj->index[i][j]], obj->verts[obj->index[i][j+1]]));
+		v2= mav_vectorNormalize(mav_vectorSub(obj->verts[obj->index[i][j+2]], obj->verts[obj->index[i][j+1]]));	    
+	      }
+	      j++;
+	    } while (fabs(mav_vectorDotProduct(v1, v2))==1.0);
 
-	  obj->surfnorm[i]= mav_vectorNormalize(mav_vectorCrossProduct(v2, v1));
-	} 
-	else 
-	{
-	  if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: face with less than 3 sides\n");
-	  obj->refs[i]=0;
-	  obj->surfnorm[i]= MAV_NULL_VECTOR;
+	    obj->surfnorm[i]= mav_vectorNormalize(mav_vectorCrossProduct(v2, v1));
+	  } 
+	  else 
+	  {
+	    if (mav_opt_output==MAV_VERBOSE) mavlib_ac3d_fprintf("Warning: face with less than 3 sides.\n");
+	    obj->refs[i]=0;
+	    obj->surfnorm[i]= MAV_NULL_VECTOR;
+	  }
 	}
       }
 
@@ -508,7 +563,7 @@ int mavlib_ac3d_parseObject(MAV_facet *fc, MAV_polyline *pl, MAV_matrix mat)
 	if (found) obj->vertnorm[i]= mav_vectorNormalize(obj->vertnorm[i]);
       }
 
-      /* create new facet for each surface */
+      /* create new facet or polyline for each surface */
       for (i=0; i<obj->numsurfs; i++) {
 	if (obj->refs[i]>0) {
 	  if ((obj->surfprops[i] & 0xF)==0) 
@@ -560,18 +615,10 @@ int mavlib_ac3d_parseObject(MAV_facet *fc, MAV_polyline *pl, MAV_matrix mat)
 	    {
 	      pl->closed[pl->nlines]= MAV_FALSE;
 	    }
+
 	    for (j=0; j<pl->np[pl->nlines]; j++) pl->vert[pl->nlines][j]= obj->verts[obj->index[i][j]];
 
-	    /* textured or not */
-	    if (obj->texture[0]) 
-	    {
-	      pl->sp[pl->nlines]= mav_surfaceParamsNew(MAV_LIT_TEXTURE, 0, mavlib_ac3d_matLookUp[obj->matindex[i]], mavlib_ac3d_texLookUp(obj->texture));
-	      if (pl->sp[pl->nlines]->texture==-1) return -1;
-	    }
-	    else
-	    {
-	      pl->sp[pl->nlines]= mav_surfaceParamsNew(MAV_MATERIAL, 0, mavlib_ac3d_matLookUp[obj->matindex[i]], 0);
-	    }
+	    pl->sp[pl->nlines]= mav_surfaceParamsNew(MAV_COLOUR, mavlib_ac3d_colLookUp[obj->matindex[i]], 0, 0);
 	    pl->nlines++;
 	  }
 	}
@@ -622,6 +669,8 @@ int mavlib_compositeAC3DRead(char *filename, MAV_composite *c, MAV_matrix m)
   char str[100];
   int ns, sp, nolines=0, nosurf=0, otbk;
 
+  mavlib_ac3d_nl=0;
+
   /* open file */
   if (!mavlib_ac3d_parseOpen(filename)) {
     if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: can not open AC3D file %s\n", filename);
@@ -631,7 +680,7 @@ int mavlib_compositeAC3DRead(char *filename, MAV_composite *c, MAV_matrix m)
   /* read version string */
   mavlib_ac3d_parseString(str);
   if (!strstr(str, "AC3D")) {
-    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: file does not start with ACD3 preamble\n");
+    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: file does not start with ACD3 preamble.\n");
     return MAV_FALSE;
   }
 
@@ -651,11 +700,11 @@ int mavlib_compositeAC3DRead(char *filename, MAV_composite *c, MAV_matrix m)
   {
     if (mavlib_ac3d_source)
     {
-      fprintf(stderr, "Parsing AC3D file %s - No. surfaces: %i", filename, nosurf);
+      fprintf(stderr, "Parsing AC3D file %s - No. surfaces: %i...", filename, nosurf);
     }
     else
     {
-      fprintf(stderr, "Parsing AC3D buffer - No. surfaces: %i", nosurf);
+      fprintf(stderr, "Parsing AC3D buffer - No. surfaces: %i...", nosurf);
     }
   }
 
@@ -684,14 +733,14 @@ int mavlib_compositeAC3DRead(char *filename, MAV_composite *c, MAV_matrix m)
   /* close it all down and open the file again */
   mavlib_ac3d_parseClose();
   if (!mavlib_ac3d_parseOpen(filename)) {
-    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: can not open AC3D file %s\n", filename);
+    if (mav_opt_output==MAV_VERBOSE) mavlib_ac3d_fprintf("Error: can not open the AC3D file the second time around.\n");
     return MAV_FALSE;
   }
     
   /* read version string */
   mavlib_ac3d_parseString(str);
   if (!strstr(str, "AC3D")) {
-    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "\nError: file does not start with ACD3 preamble\n");
+    if (mav_opt_output==MAV_VERBOSE) mavlib_ac3d_fprintf("Error: file does not start with ACD3 preamble the second time around.\n");
     return MAV_FALSE;
   }
   
@@ -735,7 +784,7 @@ int mavlib_compositeAC3DRead(char *filename, MAV_composite *c, MAV_matrix m)
   /* calculate and store its BB */
   mav_compositeCalcBB(c);
 
-  if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, " Done\n");
+  if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, " done.\n");
   if (mav_opt_compositeSetMatrix) c->matrix= MAV_ID_MATRIX;
   
   return MAV_TRUE;
@@ -751,11 +800,13 @@ int mav_compositeReadAC3D(char *filename, MAV_composite *c, MAV_matrix m)
 
   mavlib_ac3d_source=1;
   mavlib_ac3d_matLookUp= mav_malloc(mav_opt_maxMaterials*sizeof(int));
+  mavlib_ac3d_colLookUp= mav_malloc(mav_opt_maxColours*sizeof(int));
 
   rv= mavlib_compositeAC3DRead(filename, c, m);
   c->filename= strdup(filename);
 
   mav_free(mavlib_ac3d_matLookUp);
+  mav_free(mavlib_ac3d_colLookUp);
 
   return rv;
 }
@@ -770,10 +821,12 @@ int mav_compositeReadAC3DBuf(char *buf, MAV_composite *c, MAV_matrix m)
   
   mavlib_ac3d_source=0;
   mavlib_ac3d_matLookUp= mav_malloc(mav_opt_maxMaterials*sizeof(int));
+  mavlib_ac3d_colLookUp= mav_malloc(mav_opt_maxColours*sizeof(int));
 
   rv= mavlib_compositeAC3DRead(buf, c, m);
 
   mav_free(mavlib_ac3d_matLookUp);
+  mav_free(mavlib_ac3d_colLookUp);
 
   return rv;
 }
