@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -53,7 +53,7 @@ void mavlib_updateSplash(MAV_facet *f, MAV_text *t, float time)
   if (time < 1.0)
   {
     val= sin(MAV_PI_OVER_2*time);
-    f->matrix= mav_matrixSet(360.0*val,270.0+90.0*val,180.0*(1.0-val),0.0,0.0,
+    f->matrix= mav_matrixSetOld(360.0*val,270.0+90.0*val,180.0*(1.0-val),0.0,0.0,
                              8.0*(1.0-val));
     t->matrix= mav_matrixSet(0.0,0.0,0.0, 0.0,-2.0,6.0*(1.0-val));
     t->matrix= mav_matrixScaleSet(t->matrix, 0.65);
@@ -446,63 +446,37 @@ int mav_linePolygonIntersection(MAV_polygon *apoly, MAV_line ln, MAV_objectInter
   MAV_vector *new_vert, axis, norm;
   MAV_matrix rot;
   MAV_line rotln;
-  float ca, sa;
+  float ca, sa, ang;
   int i, j, c=MAV_FALSE;
 
 /* Malloc off space to rotate vertices so norm is parrallel to Z axis */
 
   new_vert=mav_malloc(apoly->np*sizeof(MAV_vector));
 
-/* Find rotation and translation so polygon's normal is aligned with Z axis */
+/* Find rotation so polygon's normal is aligned with Z axis */
 
-  if (apoly->norm.x == 0.0 && apoly->norm.y == 0.0) 
-  {
-    axis.x = 1.0;
-    axis.y = 0.0;
-    axis.z = 0.0;
-  }
-  else
-  {
-    axis.x = -apoly->norm.y;
-    axis.y = apoly->norm.x;
-    axis.z = 0.0;
-  }  
-  
-  axis= mav_vectorNormalize(axis);
-
-  ca = apoly->norm.z;
-
-  /* Numerical precision and polygon's normal normalized issue */
+  ca= apoly->norm.z;
   if (ca>=1.0)
   {
-    ca=1.0;
-    sa=0.0;
+    rot= MAV_ID_MATRIX;
+  }
+  else if (ca<=-1.0)
+  {
+    rot= mav_matrixSet(0,180,0,0,0,0);
   }
   else
   {
-    sa = sqrt(1.0 - ca*ca);
+    axis.x= apoly->norm.y;
+    axis.y= -apoly->norm.x;
+    axis.z= 0;
+
+    ang= MAV_RAD2DEG(acos(ca));
+    sa= mav_vectorMag(axis);
+    if (sa<0.0) ang= 360-ang;
+
+    rot= mav_matrixQuaternionConvert(mav_quaternionSet(mav_vectorNormalise(axis), ang));
   }
 
-  rot.mat[0][0] = axis.x*axis.x+ca*(1.0-axis.x*axis.x);
-  rot.mat[1][0] = axis.x*axis.y*(1.0-ca)+axis.z*sa;
-  rot.mat[2][0] = axis.z*axis.x*(1.0-ca)-axis.y*sa;
-  rot.mat[3][0] = 0.0;
-
-  rot.mat[0][1] = axis.x*axis.y*(1.0-ca)+axis.z*sa;
-  rot.mat[1][1] = axis.y*axis.y+ca*(1.0-axis.y*axis.y);
-  rot.mat[2][1] = axis.y*axis.z*(1.0-ca)+axis.x*sa;
-  rot.mat[3][1] = 0.0;
-
-  rot.mat[0][2] = axis.z*axis.x*(1.0-ca)+axis.y*sa;
-  rot.mat[1][2] = axis.y*axis.z*(1.0-ca)-axis.x*sa;
-  rot.mat[2][2] = axis.z*axis.z+ca*(1.0-axis.z*axis.z);
-  rot.mat[3][2] = 0.0;
-
-  rot.mat[0][3] = 0.0;
-  rot.mat[1][3] = 0.0;
-  rot.mat[2][3] = 0.0;
-  rot.mat[3][3] = 1.0;
-
 /* Rotate all the polygon pts to be Z axis aligned */
 
   for (i=0; i<apoly->np; i++) new_vert[i]= mav_vectorMult(apoly->vert[i], rot);
