@@ -21,7 +21,7 @@
  The authors can be contacted via:
  www   - http://aig.cs.man.ac.uk
  email - maverik@aig.cs.man.ac.uk
- mail  - Advanced Interfaces Group, Room 2.90, Computer Science Building, 
+ mail  - Advanced Interfaces Group, Room 2.94, Computer Science Building, 
          University of Manchester, Manchester, M13 9PL, UK
 */
 
@@ -32,6 +32,8 @@
 #include <stdlib.h>
 #include <dlfcn.h>
 #include <string.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 int mavlib_defaultColours[][3]={
 {148,   0, 211},             /* darkviolet */
@@ -82,6 +84,7 @@ int mavlib_defaultColours[][3]={
 
 extern int mavlib_texNum;
 extern unsigned long mavlib_tex[][16384];
+extern int mavlib_voodoo;
 
 int mav_opt_noWins=MAV_FALSE;
 int mav_opt_stereo=0;
@@ -96,11 +99,7 @@ int mav_opt_right_y=-1;
 int mav_opt_right_width=-1;
 int mav_opt_right_height=-1;
 char *mav_opt_right_name=NULL;
-#ifdef MAV_VOODOO
-int mav_opt_restrictMouse= MAV_TRUE;
-#else
-int mav_opt_restrictMouse= MAV_FALSE;
-#endif
+int mav_opt_restrictMouse=-1;
 MAV_class *mav_class_world;
 MAV_class *mav_class_any;
 MAV_class *mav_class_none;
@@ -229,8 +228,6 @@ int mavlib_restrictMouseOpt=0;
 
 void mavlib_cf1(void)
 {
-#ifdef MAV_LINUX
-#ifdef MAV_VOODOO
   MAV_window *o, *w;
   
   mavlib_fullscreen=!mavlib_fullscreen;
@@ -254,8 +251,6 @@ void mavlib_cf1(void)
   }
   
   mav_windowSet(o);
-#endif
-#endif
 }
 
 
@@ -347,6 +342,8 @@ char *mav_windowsModuleID(void)
 
 int mav_windowsModuleInit(void)
 {
+  FILE *f;
+  char buf[500], sbuf[510], filename[500];
   int lx, ly, lw=0, lh=0, rx, ry, rw, rh, i;
 
   /* add the new module */  
@@ -389,12 +386,10 @@ int mav_windowsModuleInit(void)
     mav_ctrlF_desc[i]=NULL;
   }
 
-#ifdef MAV_LINUX
-#ifdef MAV_VOODOO
-  mav_ctrlF[1]= mavlib_cf1;
-  mav_ctrlF_desc[1]= "Ctrl-F1 toggle between full screen and in-window rendering";
-#endif
-#endif
+  if (mavlib_voodoo) {
+    mav_ctrlF[1]= mavlib_cf1;
+    mav_ctrlF_desc[1]= "Ctrl-F1 toggle between full screen and in-window rendering";
+  }
 
   mav_ctrlF[12]= mavlib_cf12;
   mav_ctrlF_desc[12]= "Ctrl-F12 load a module on the fly";
@@ -421,15 +416,18 @@ int mav_windowsModuleInit(void)
     }
     else
     {
-#ifdef MAV_VOODOO
-      ly= 0;
-#else
+      if (mavlib_voodoo) 
+      {
+	ly= 0;
+      }
+      else
+      {
 #ifdef MAV_LINUX
-      ly= mav_yres/2-35;
+	ly= mav_yres/2-35;
 #else
-      ly= mav_yres/2;
+	ly= mav_yres/2;
 #endif      
-#endif
+      }
     }
 
     if (mav_opt_width>=0)
@@ -438,11 +436,14 @@ int mav_windowsModuleInit(void)
     }
     else
     {
-#ifdef MAV_VOODOO
-      lw= 640;
-#else
-      lw= mav_xres/2.0;
-#endif
+      if (mavlib_voodoo)
+      {
+	lw= 640;
+      }
+      else
+      {
+	lw= mav_xres/2.0;
+      }
     }
 
     if (mav_opt_height>=0)
@@ -451,11 +452,14 @@ int mav_windowsModuleInit(void)
     }
     else
     {
-#ifdef MAV_VOODOO
-      lh= 480;
-#else
-      lh= mav_yres/2.0;
-#endif
+      if (mavlib_voodoo) 
+      {
+	lh= 480;
+      }
+      else
+      {
+	lh= mav_yres/2.0;
+      }
     }
   
     if (mav_opt_fullscreen==MAV_TRUE) {
@@ -480,11 +484,18 @@ int mav_windowsModuleInit(void)
     }
     else
     {
-#ifdef MAV_VOODOO
-      ry=0;
+      if (mavlib_voodoo)
+      {
+	ry=0;
+      }
+      else
+      {
+#ifdef MAV_LINUX
+	ry= mav_yres/2-35;
 #else
-      ry= mav_yres/2;
+	ry= mav_yres/2;
 #endif
+      }
     }
 
     if (mav_opt_right_width>=0)
@@ -493,11 +504,14 @@ int mav_windowsModuleInit(void)
     }
     else
     {
-#ifdef MAV_VOODOO
-      rw= 640;
-#else
-      rw= mav_xres/2.0;;
-#endif
+      if (mavlib_voodoo) 
+      {
+	rw= 640;
+      }
+      else
+      {
+	rw= mav_xres/2.0;
+      }
     }
 
     if (mav_opt_right_height>=0)
@@ -506,11 +520,14 @@ int mav_windowsModuleInit(void)
     }
     else
     {
-#ifdef MAV_VOODOO
-      rh= 480;
-#else
-      rh= mav_yres/2.0;
-#endif
+      if (mavlib_voodoo) 
+      {
+	rh= 480;
+      }
+      else
+      {
+	rh= mav_yres/2.0;
+      }
     }
   
     if (mav_opt_fullscreen==MAV_TRUE) {
@@ -520,6 +537,27 @@ int mav_windowsModuleInit(void)
       rh= mav_yres;
     }
 
+    /* find name of executable */
+    sprintf(filename, "/tmp/mavname%i", getpid());
+#ifdef MAV_LINUX
+    sprintf(buf, "ps a | awk '{if ($1==%i) print $5}' 2>&1 >%s", getpid(), filename);
+#else
+    sprintf(buf, "ps -f | awk '{if ($2==%i) print $8}' 2>&1 >%s", getpid(), filename);
+#endif
+    system(buf);
+    f= fopen(filename, "r");
+    if (f) 
+    {
+      if (fscanf(f, "%s", buf)!=1) strcpy(buf, "Maverik");
+      fclose(f);
+    }
+    else
+    {
+      strcpy(buf, "Maverik");
+    }
+    sprintf(sbuf, "rm -f %s", filename);
+    system(sbuf);
+
     /* open the windows with the correct name */
     if (mav_opt_stereo) 
     {
@@ -529,12 +567,11 @@ int mav_windowsModuleInit(void)
       }
       else
       {
-	mav_win_left= mav_windowNew(lx, ly, lw, lh, "Maverik: left");
+	sprintf(sbuf, "%s (left)", buf);
+	mav_win_left= mav_windowNew(lx, ly, lw, lh, sbuf);
       }
 
-#ifdef MAV_VOODOO
-      mav_gfx3DfxBoardSet(1);
-#endif
+      if (mavlib_voodoo) mav_gfx3DfxBoardSet(1);
 
       if (mav_opt_right_name) 
       {
@@ -542,7 +579,8 @@ int mav_windowsModuleInit(void)
       }
       else
       {
-	mav_win_right= mav_windowNew(rx, ry, rw, rh, "Maverik: right");  
+	sprintf(sbuf, "%s (right)", buf);
+	mav_win_right= mav_windowNew(rx, ry, rw, rh, sbuf);  
       }
 
       /* Set correct view */
@@ -558,7 +596,7 @@ int mav_windowsModuleInit(void)
       }
       else
       {
-	mav_win_left= mav_windowNew(lx, ly, lw, lh, "Maverik: mono");
+	mav_win_left= mav_windowNew(lx, ly, lw, lh, buf);
       }
     }
 
@@ -566,13 +604,20 @@ int mav_windowsModuleInit(void)
     mav_win_mouse= mav_win_left;
   }
 
-  /* Restrict mouse to left window if requested */
-
-  if (mav_opt_restrictMouse) {
-    mavlib_restrictLastX= lw/2;
-    mavlib_restrictLastY= lh/2;
-    mav_frameFn1Add(mavlib_restrictMouse);
+  /* Restrict mouse to current window if requested */
+  if (mav_opt_restrictMouse == -1) {
+    if (mavlib_voodoo)
+    {
+      mav_opt_restrictMouse= MAV_TRUE;
+    }
+    else
+    {
+      mav_opt_restrictMouse= MAV_FALSE;
+    }
   }
+  mavlib_restrictLastX= lw/2;
+  mavlib_restrictLastY= lh/2;
+  mav_frameFn1Add(mavlib_restrictMouse);
 
   /* Set default lighting model, light and material (now we have a valid gfx context) */
   if (mav_win_left) {
@@ -606,6 +651,10 @@ int mav_windowsModuleInit(void)
 
   mav_sp_default= mav_surfaceParamsNew(MAV_MATERIAL, 0, 1, 0);
 
+/* Define the mouse's surface params */
+
+  mav_mouseSurfaceParamsSet(mav_surfaceParamsNew(MAV_COLOUR, MAV_COLOUR_RED, 0, 0));
+
   return 1;
 }
 
@@ -673,6 +722,3 @@ void mav_eyeRight(MAV_window *w)
 
   w->eye= mav_vectorAdd(w->vp->trans_eye, mav_vectorScalar(w->right, w->vmp->offset*0.5));
 }
-
-
-
